#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Fill the path, using the nonzero winding number rule to determine the region
to fill (see 8.5.3.3.2, "Nonzero Winding Number Rule"). Any subpaths that
are open shall be implicitly closed before being filled.
"""
